/* 
 * $Id: XDTPTransform.h 20876 2009-04-12 20:04:59Z ozawa $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 20876 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#ifndef ___XDTPTRANSFORM_H
#define ___XDTPTRANSFORM_H

#include "XDTPObjectWrapper.h"
#include "modules/XDTPModuleLoader.h"

#include "libxslt/transform.h"

namespace XDTP {

typedef enum {
    OBJECT_TYPE_USTRING = 0,
    OBJECT_TYPE_XSL,
    OBJECT_TYPE_MC,
    OBJECT_TYPE_XDTP_EXT_MODULE,    // 外部確保モジュール(解放不要)
    OBJECT_TYPE_XDTP_EXT_CHECKER,   // 外部確保モジュール(解放不要)
    OBJECT_TYPE_XDTP_LCL_MODULE,    // 内部確保モジュール(解放要)
    OBJECT_TYPE_XDTP_LCL_CHECKER,   // 内部確保モジュール(解放要)
} OBJECT_TYPES;

typedef struct {
    XDTPModule  *module;
    XDTPChecker *checker;
} XDTP_MC;

typedef std::list<XDTPModuleLibrary*>   LibraryList;

class XDTPTransform
{
public:
    XDTPTransform();
    virtual ~XDTPTransform();
    
    /**
     * 初期化処理を行います。
     * 初期化後に set/get 系のメソッドを使用することは避けてください。
     * 変換処理中に set/get 系メソッドを使用することはできません。
     * 
     * @return 初期化に成功した場合は true を戻します。
     */
    bool initialize();
    
    /**
     * オブジェクトを解放する準備を行います。
     * 変換処理後、オブジェクトを利用することがない場合や、状態を変更する
     * 場合は、事前に呼び出す必要があります。
     */
    void uninitialize();
    
    /**
     * 変換処理を実行します。
     * 
     * @param anInput 変換元XMLファイル
     * @param anOutput 出力先ファイル（既存のファイルは上書きします）
     * @return 変換に成功した場合は true を戻します。
     */
    bool transform(const Glib::ustring& anInput, const Glib::ustring& anOutput);
    
    /**
     * 変換処理時に発生したエラーメッセージを出力します。
     * エラーが発生しなかった場合は、何も表示しません。
     * 
     * @param out 出力先
     */
    void showErrorMsg(FILE* out = stdout) const;
    
    /**
     * XML文章の正当性チェックをスキップします。
     * 
     * @param bCheck スキップする場合は true を指定します。
     */
    void skipXMLCheck(bool bCheck);
    
    /**
     * XML文章の正当性チェックをスキップするか調べます。
     * 
     * @return スキップする場合は true を戻します。
     */
    bool isXMLCheck() const;
    
    /**
     * 変換用XSLファイルを追加します。
     * 
     * @param aPath XSLファイルへのパス
     * @param useOutputType ファイルが出力形式を保持し、これを利用する場合は true を指定します。
     */
    void addXSLFile(const Glib::ustring& aPath, bool useOutputType);
    
    /**
     * 変換用XSLオブジェクトを追加します。
     * 
     * @param aXSL XSLオブジェクト
     * @param useOutputType オブジェクトが出力形式を保持し、これを利用する場合は true を指定します。
     */
    void addXSLObject(xsltStylesheetPtr aXSL, bool useOutputType);
    
    /**
     * 変換用XSLファイルの一覧をクリアします。
     */
    void clearXSLFile();
    
    /**
     * 変換用XSLファイルの一覧を取得します。
     * 
     * @deprecated
     * @return 変換用XSLファイルの一覧
     */
    const StringList& getXSLFileList();

    /**
     * 変換用XSLファイルおよびオブジェクトの一覧を取得します。
     * 
     * @return 変換用XSLファイルおよびオブジェクトの一覧
     */
    const XDTPObjectWrapper& getXSLList() const;

    /**
     * 変換用XDTPモジュールを追加します。
     * 
     * @param aPath XDTPモジュールへのパス
     */
    void addXDTPModule(const Glib::ustring& aPath);

    /**
     * 変換用XDTPモジュールを追加します。
     * 
     * @param aModule XDTPモジュール
     * @param aChecker XDTPチェッカ。使用しない場合は NULL を指定できます。
     */
    void addXDTPModule(XDTPModule *aModule, XDTPChecker *aChecker);

    /**
     * 変換用XDTPモジュールをクリアします。
     */
    void clearXDTPModule();
    
    /**
     * 変換用XDTPモジュールの一覧を取得します。
     * 
     * @deprecated
     * @return 変換用XDTPモジュールの一覧
     */
    const StringList& getXDTPModuleList();

    /**
     * 変換用XDTPモジュールの一覧を取得します。
     * 
     * @return 変換用XDTPモジュールの一覧
     */
    const XDTPObjectWrapper& getXDTPMCList() const;

    /**
     * 変換用XDTPモジュールへのオプションを追加します。
     * 
     * @param anOption 追加オプション
     */
    void addOption(const Glib::ustring& anOption);

    /**
     * 変換用XDTPモジュールへのオプションをクリアします。
     */
    void clearOption();
    
    /**
     * 変換用XDTPモジュールへのオプションの一覧を取得します。
     * 
     * @return 変換用XDTPモジュールへのオプションの一覧
     */
    const StringList& getOptionList() const;

    /**
     * 出力形式を指定します。
     * 
     * @param aType 出力形式
     */
    void setOutputType(const Glib::ustring& aType);

    /**
     * 出力形式を取得します。
     * 
     * @return 出力形式
     */
    const Glib::ustring& getOutputType() const;

    /**
     * メッセージの表示レベルを指定します。
     * 
     * @param aMode メッセージの表示レベル
     */
    void setVerboseMode(int aMode);
    
    /**
     * メッセージの表示レベルを取得します。
     */
    int getVerboseMode() const;

    /**
     * 入力ファイルのエンコードを指定します。
     * 
     * @param anEncoding 入力ファイルのエンコード
     */
    void setInputEncoding(const Glib::ustring& anEncoding);

    /**
     * 入力ファイルのエンコードを取得します。
     * 
     * @return 入力ファイルのエンコード
     */
    const Glib::ustring& getInputEncoding() const;
    
    /**
     * 出力ファイルのエンコードを指定します。
     * 
     * @param anEncoding 出力ファイルのエンコード
     */
    void setOutputEncoding(const Glib::ustring& anEncoding);

    /**
     * 出力ファイルのエンコードを取得します。
     * 
     * @return 出力ファイルのエンコード
     */
    const Glib::ustring& getOutputEncoding() const;

public:
    /**
     * エラーメッセージを設定します。
     * 
     * @param aMsg エラーメッセージ
     */
    void setErrorMsg(const Glib::ustring& aMsg);

private:
    bool loadModules(LibraryList& libraries,
                     XDTPObjectWrapper& modules, XDTPObjectWrapper& checkers);
    void unloadModules(LibraryList& libraries,
                       XDTPObjectWrapper& modules, XDTPObjectWrapper& checkers);
    bool loadLocalModule(const Glib::ustring& module,
                         XDTPObjectWrapper& modules, XDTPObjectWrapper& checkers);

    xsltStylesheetPtr parseXSL(int pos);

private:
    XDTPObjectWrapper   m_XSLObjects;
    int                 m_XSL4OutputPos;
    XDTPObjectWrapper   m_XDTPModules;
    StringList          m_OptionList;
    bool                m_bSkipXMLCheck;
    int                 m_nVerbose;
    Glib::ustring       m_OutputType;
    Glib::ustring       m_InputEncoding;
    Glib::ustring       m_OutputEncoding;
    mode_t				m_OutputMask;
    
    Glib::ustring       m_ErrorMsg;

    LibraryList         m_libraries;
    XDTPObjectWrapper   m_modules;
    XDTPObjectWrapper   m_checkers;

private:
    StringList      __m_XSLFileList;    // deprecated
    StringList      __m_XDTPModuleList; // deprecated
};

};

#endif
