/* 
 * $Id: XMLTool.h 20876 2009-04-12 20:04:59Z ozawa $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 20876 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#ifndef ___XMLTOOL_H
#define ___XMLTOOL_H

#include "libxml/tree.h"
#include "libxml/xpath.h"
#include "libxslt/xslt.h"
#include "libxslt/transform.h"

namespace XDTP {

class XMLTool
{
public:
	XMLTool();
	virtual ~XMLTool();

    /**
     * 複数階層に跨るElementを一度に生成します。
     * 既存のElementが存在する場合は、存在しないElementのみを生成します。
     * 
     * @param aDocument
     *            生成対象のDOM オブジェクト
     * @param aXPath
     *            複数階層に跨るElementを識別するXPath。
     *            １つの階層に複数のElementを指定するようなことはできません。(書式: /e1/e2/e3/e4 )
     * @return 作成に成功したときは最下層の Element。失敗したときは null。
     */
    xmlNodePtr createElements(xmlDocPtr aDocument, const Glib::ustring& aXPath);
    
    /**
     * XMLファイルのエンコーディングを取得します。
     * 
     * @param aFile XMLファイル名
     * @return エンコーディング
     */
    std::string getEncoding(const Glib::ustring& aFile);

    /**
     * Nodeからの相対XPath指定で得られるNodeの一覧を取得します。
     * 
     * @param aNode
     *            探索元のNode
     * @param aXPath
     *            Nodeからの相対XPath
     * @return 取得したNodeの一覧を保持するオブジェクト。
     *         不要になった段階で、xmlXPathFreeObjectする必要があります。
     */
    xmlXPathObjectPtr getNodeList(xmlNodePtr aNode, const Glib::ustring& aXPath);

    /**
     * Nodeの直下に存在するTextを取得します。
     * 
     * @param aNode
     *            Textを含んだNode
     * @return 取得したTextの文字列。Textが存在しない場合は空文字列。
     */
    Glib::ustring getText(xmlNodePtr aNode);

    /**
     * NodeがAttributeかチェックします。
     * 
     * @param aNode
     *            チェック対象のNode
     * @return Attributeの場合は true
     */
    bool isAttribute(xmlNodePtr aNode);

    /**
     * NodeがCommentかチェックします。
     * 
     * @param aNode
     *            チェック対象のNode
     * @return Commentの場合は true
     */
    bool isComment(xmlNodePtr aNode);

    /**
     * NodeがElementかチェックします。
     * 
     * @param aNode
     *            チェック対象のNode
     * @return Elementの場合は true
     */
    bool isElement(xmlNodePtr aNode);

    /**
     * NodeがTextかチェックします。
     * 
     * @param aNode
     *            チェック対象のNode
     * @return Textの場合は true
     */
    bool isText(xmlNodePtr aNode);

    /**
     * Nodeの直下にTextを設定します。
     * 
     * @param aNode
     *            Textを設定するNode
     * @param aText
     *            設定するText
     * @return 設定に成功した場合は true
     */
    bool setText(xmlNodePtr aNode, const Glib::ustring& aText);

    /**
     * DOMドキュメントを、XSLTを用いて変換します。
     * 
     * @param aDocument
     *            変換元のDOMドキュメント
     * @param aXSLFile
     *            XSLファイル
     * @return 変換後のDOMドキュメント。不要になった段階でxmlFreeDocする必要があります。
     */
    xmlDocPtr transform(xmlDocPtr aDocument, const Glib::ustring& aXSLFile);

    /**
     * DOMドキュメントを、XSLTを用いて変換します。
     * 
     * @param aDocument
     *            変換元のDOMドキュメント
     * @param aXSL
     *            XSLオブジェクト
     * @return 変換後のDOMドキュメント。不要になった段階でxmlFreeDocする必要があります。
     */
    xmlDocPtr transform(xmlDocPtr aDocument, xsltStylesheetPtr aXSL);
};

};

#endif
