/* 
 * $Id: GLSDImportModuleAdapter.h 34706 2013-08-05 06:24:20Z ozawa $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 34706 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#ifndef ___GLSDIMPORTMODULEADAPTER_H
#define ___GLSDIMPORTMODULEADAPTER_H

#include "modules/imports/GLSDImportModule.h"

namespace XDTP {

typedef enum {
    TIP_SUCCESS = 0,
    TIP_FAILED = 1,
    TIP_SKIP = 2
} TIP_STATUS;

class GLSDImportModuleAdapter : public GLSDImportModule
{
public:
	GLSDImportModuleAdapter();
	virtual ~GLSDImportModuleAdapter();

    /**
     * 処理を開始する前に出力形式を設定します。
     * 
     * @param aType 出力形式
     */
    virtual void setOutputType(const Glib::ustring& aType);
    
    /**
     * 処理を開始する前にオプションを設定します。
     * 
     * @param anOptions オプション
     */
    virtual void setOption(const StringList& anOptions);

    /**
     * 処理を開始する前に出力ファイル名を設定します。
     * 
     * @param anOutputFile 出力ファイル名
     */
    virtual void setOutputFile(const Glib::ustring& anOutputFile);

    /**
     * 処理を開始する前に変換元XMLファイル名を設定します。
     * 
     * @param aXMLFile 変換元XMLファイル名
     */
    virtual void setXMLFile(const Glib::ustring& aXMLFile);
    
    /**
     * 入力ファイルのエンコードを指定します。
     * XMLファイルはこの限りではありません。
     * 
     * @param anInputEncoding 入力ファイルのエンコード
     */
    virtual void setInputEncoding(const Glib::ustring& anInputEncoding);

    /**
     * 出力ファイルのエンコードを指定します。
     * 
     * @param anOutputEncoding 出力ファイルのエンコード
     */
    virtual void setOutputEncoding(const Glib::ustring& anOutputEncoding);

    /**
     * 現在のXDTPTransformモジュールを指定します。
     * 
     * @param aXDTPTransform 現在のXDTPTransformモジュール
     */
    virtual void setXDTPTransform(const XDTPTransform *aXDTPTransform);

    /**
     * XMLファイルのエンコーディングを指定します。
     * 
     * @param anEncoding XMLファイルのエンコーディング
     */
    virtual void setXMLEncoding(const std::string& anEncoding);

    /**
     * ファイルをparseする直前に呼び出されます。
     * 
     * @param aFile parse対象のファイル名
     */
    virtual void treatPreFile(const Glib::ustring& aFile);

    /**
     * parseしたXML文章をXSL変換する直前に呼び出されます。
     * 
     * @param aDocument XML文章
     */
    virtual void treatPreDocument(xmlDocPtr aDocument);

    /**
     * XSL変換後のXML文章を保存する直前に呼び出されます。
     * 
     * @param aDocument XML文章
     */
    virtual void treatPostDocument(xmlDocPtr aDocument);

    /**
     * XML文章を保存した直後に呼び出されます。
     * 
     * @param aFile 保存したファイル名
     */
    virtual void treatPostFile(const Glib::ustring& aFile);
    
protected:
    /**
     * 現在のXDTPTransformモジュールを取得します。
     * 
     * @return 現在のXDTPTransformモジュール
     */
    virtual const XDTPTransform * getXDTPTransform();

protected:
    /**
     * ファイルからインポート処理対象の部分を抽出し、置換処理を
     * 行う際に呼び出されます。
     * 
     * @param aRef インポート先のファイル
     * @param aCaption キャプション
     * @param anEncoding インポート先のファイルのエンコード
     * @param aNewData 置換すべき文字列を応答します。
     * @return 処理結果を応答します。
     */
    virtual TIP_STATUS transformImportPointCallback
        (const Glib::ustring& aRef, const Glib::ustring& aCaption,
         const Glib::ustring& anEncoding, Glib::ustring& aNewData);

    /**
     * インポート処理対象を識別するノードを構築します。
     * 
     * @param aType 処理対象の種類
     * @param aRef インポート先のファイル
     * @param aCaption キャプション
     * @param anEncoding インポート先のファイルのエンコード
     * @return インポート処理対象を識別するノード
     */
    xmlNodePtr createNewImportPointNode
         (const Glib::ustring& aType, const Glib::ustring& aRef,
          const Glib::ustring& aCaption, const Glib::ustring& anEncoding);
    
    /**
     * ファイルからインポート処理対象の部分を抽出し、置換処理を行います。
     * 
     * @param aFile 処理対象のファイル
     * @param aType 処理対象の種類
     * @return 成功した場合は true を戻します。
     */
    bool transformImportPoint
        (const Glib::ustring& aFile, const Glib::ustring& aType);

protected:
    /**
     * URIで参照するリソースへのIOChannelを取得します。
     * 取得できない場合は XDTP::Exception を throw します。
     * 
     * @param aRef URI
     * @param aTmpFile 一時ファイルを生成した場合は、一時ファイルへのパスを応答します。
     *                 生成しない場合は、空文字列を応答します。
     * @return リソースへのIOChannel
     */
    Glib::RefPtr<Glib::IOChannel> getIOChannel
        (const Glib::ustring& aRef, std::string& aTmpFile);

    /**
     * ローカルファイルを参照するURLを絶対パスへ変換します。
     * 
     * @param url リソースへのURL
     * @return 修正後のURL
     */
    Glib::ustring normalizeURL(const Glib::ustring& url);
    
    /**
     * ファイルの読み込み用エンコードを適切に設定する。
     * 
     * @param anIOC エンコードを設定するIOChannel
     * @param anEncoding 優先的に設定するエンコード。不明な場合は空文字列を指定する。
     */
    void setEncoding(Glib::RefPtr<Glib::IOChannel>& anIOC,
                     const Glib::ustring& anEncoding);

    /**
     * 指定のキーで識別されるオプションの値を取得します。
     * 
     * @param aKey オプションを一意に識別する値
     * @param aValue 取得したオプションの値を格納する領域
     * @return オプションが見付かり正しく取得できた場合は true を戻します。
     */
    bool getOptionInt(const Glib::ustring& aKey, int &aValue);
    bool getOptionString(const Glib::ustring& aKey, Glib::ustring& aValue);

protected:
    /**
     * 任意タイプのスタイルシートを取得する。
     * 
     * @param aDocument
     *            XHTMLドキュメント
     * @param aType
     *            スタイルシートのタイプ
     *            常に小文字で指定する必要があります。
     * @return  取得したスタイルシート
     */
    static xmlNodePtr getStyleSheet(xmlDocPtr aDocument, const Glib::ustring& aType);

    /**
     * Importタグの各種属性値を取得します。
     * 
     * @param aNode Importノード
     * @param aRef リソースのURL
     * @param aCaption キャプション
     * @param anEncoding リソースのエンコーディング
     * @return 取得に成功した場合は true を戻す。
     */
    static bool getImportAttributes(xmlNodePtr aNode, Glib::ustring& aRef,
                                    Glib::ustring& aCaption, Glib::ustring& anEncoding);

    /**
     * Importタグの各種属性値を取得します。
     *
     * @param aNode Importノード
     * @param aRef リソースのURL
     * @param anId リソースの固有ID
     * @param aCaption キャプション
     * @param anEncoding リソースのエンコーディング
     * @return 取得に成功した場合は true を戻す。
     */
    static bool getImportAttributes(xmlNodePtr aNode, Glib::ustring& aRef, Glib::ustring& anId,
                                    Glib::ustring& aCaption, Glib::ustring& anEncoding);

    /**
     * XMLドキュメント用に特殊文字をエスケープします。
     * 
     * @param aString 特殊文字を含む文字列
     * @return エスケープ後の文字列
     */
    static Glib::ustring escapeString(const Glib::ustring& aString);

    /**
     * Importタグをテキスト形式で保持します。
     */
    static bool preserveImport(xmlDocPtr aDocument, xmlNodePtr aNode,
                               const Glib::ustring& aRef,
                               const Glib::ustring& aCaption,
                               const Glib::ustring& anEncoding);

protected:
    StringList      m_OptionList;
    Glib::ustring   m_OutputType;
    Glib::ustring   m_InputEncoding;
    Glib::ustring   m_OutputEncoding;
    Glib::ustring   m_XMLFile;
    Glib::ustring   m_OutputFile;
    std::string     m_XMLEncoding;

private:
    const XDTPTransform     *m_pXDTPTransform;
};

};

#endif
